#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lcd.h"

#define NR   12

typedef struct {
    int x, y, face;
    char sign;
} card;

card card_list[NR];
int nr_turned, p1, p2;
int score, game_state;
int nr_perechi;

static void play();

static void title()
{
	//Nume, Grupa
	lcd_gotoXY(19, 1);
	lcd_str("Cristea",7);
	
	lcd_gotoXY(15, 2);
	lcd_str("Stefania",8);
	
	lcd_gotoXY(26, 3);
	lcd_str("334CB",5);
	
	_delay_ms(2500);
	lcd_clear();
	
	//Titlul proiect
	lcd_gotoXY(22, 1);
	lcd_str("Memory",6);
	
	lcd_gotoXY(28, 4);
	lcd_str("Game",4);
	
	_delay_ms(2500);
	lcd_clear();
}

static void fill_cards()
{
	int i;
	
	card_list[0].x = 10 ; card_list[0].y = 1 ; card_list[0].sign = 'a' ; card_list[0].face = 0 ;
	card_list[1].x = 10 ; card_list[1].y = 3 ; card_list[1].sign = 'd' ; card_list[1].face = 0 ;
	card_list[2].x = 10 ; card_list[2].y = 5 ; card_list[2].sign = 'x' ; card_list[2].face = 0 ;
	card_list[3].x = 29 ; card_list[3].y = 1 ; card_list[3].sign = 'g' ; card_list[3].face = 0 ;
	card_list[4].x = 29 ; card_list[4].y = 3 ; card_list[4].sign = 'e' ; card_list[4].face = 0 ;
	card_list[5].x = 29 ; card_list[5].y = 5 ; card_list[5].sign = 'y' ; card_list[5].face = 0 ;
	card_list[6].x = 49 ; card_list[6].y = 1 ; card_list[6].sign = 'g' ; card_list[6].face = 0 ;
	card_list[7].x = 49 ; card_list[7].y = 3 ; card_list[7].sign = 'd' ; card_list[7].face = 0 ;
	card_list[8].x = 49 ; card_list[8].y = 5 ; card_list[8].sign = 'y' ; card_list[8].face = 0 ;
	card_list[9].x = 69 ; card_list[9].y = 1 ; card_list[9].sign = 'x' ; card_list[9].face = 0 ;
	card_list[10].x = 69 ; card_list[10].y = 3 ; card_list[10].sign = 'a' ; card_list[10].face = 0 ;
	card_list[11].x = 69 ; card_list[11].y = 5 ; card_list[11].sign = 'e' ; card_list[11].face = 0 ;
	
	for (i = 0; i < NR; i++){
		lcd_gotoXY(card_list[i].x,card_list[i].y);
		lcd_putchr('?');
	}
	
	lcd_gotoXY(0,0);
	lcd_str("Score: 0",8);
}

static void replay()
{
	lcd_clear();
	
	lcd_gotoXY(20,1);
	lcd_str("Replay?",7);
	
	lcd_gotoXY(33,3);
	lcd_str("YES",3);
	
	lcd_gotoXY(35,5);
	lcd_str("NO",2);
	
	while(1) {
		if (game_state == 3) {
			if (!(PIND & (1 << PD0))) {
				lcd_clear();
				play();
			}
				
			
			if (!(PIND & (1 << PD1)))
				lcd_clear();
		}
	}
}

static void check_pair()
{
	char sr[11];
	
	if (card_list[p1].sign == card_list[p2].sign) {
		score = score + 15;
		nr_perechi++;
		
		lcd_gotoXY(0,0);
		memset (sr,' ',11);
		sprintf(sr,"Score: %d", score);
		lcd_str(sr,strlen(sr));
		
		if(nr_perechi == 6) {
			game_state = 3;
			replay();
		}
	} else {
		score = score - 5;
		card_list[p1].face = 0;
		card_list[p2].face = 0;
		
		lcd_gotoXY(0,0);
		memset (sr,' ',11);
		sprintf(sr,"Score: %d", score);
		lcd_str(sr,strlen(sr));
		
		_delay_ms(1500);
		lcd_gotoXY(card_list[p1].x, card_list[p1].y);
		lcd_putchr('?');
		lcd_gotoXY(card_list[p2].x, card_list[p2].y);
		lcd_putchr('?');
	}
	
	nr_turned = 0;
	p1 = -1;
	p2 = -1;
}

static void change_sign(int index)
{
	lcd_gotoXY(card_list[index].x, card_list[index].y);
	
	if (card_list[index].face == 0) {
		lcd_putchr(card_list[index].sign);
		card_list[index].face = 1;
		nr_turned++;
		
		if(nr_turned==1) 
			p1 = index;
		
		if(nr_turned==2) {
			p2 = index;
			check_pair();
		}
	}
}

static void menu()
{
	lcd_clear();

	lcd_gotoXY(22, 1);
	lcd_str("Memory",6);
	
	lcd_gotoXY(20, 4);
	lcd_str("Simple",6);
}

static void change_play1()
{
	int i;
	
	game_state = 1;
	
	lcd_clear();
	fill_cards();
	
	for (i = 0; i < NR; i++){
		lcd_gotoXY(card_list[i].x,card_list[i].y);
		lcd_putchr(card_list[i].sign);
	}
	
	_delay_ms(1700);
	fill_cards();
}

static void change_play2()
{
	game_state = 1;
	
	lcd_clear();
	fill_cards();	
}

static void play()
{
	menu();
	
	nr_turned = 0;
	score = 0;
	game_state = 0;
	nr_perechi = 0;

	while(1) {
		if (!(PIND & (1 << PD0)))
			if (game_state == 0)
				change_play1(0);
		
		if (!(PIND & (1 << PD1)))
			if (game_state == 0)
				change_play2(0);
		
		if (!(PINA & (1 << PA5)))
				change_sign(0);
		
		if (!(PINA & (1 << PA6)))
				change_sign(1);
		
		if (!(PIND & (1 << PD4)))
				change_sign(2);
		
		if (!(PIND & (1 << PD5)))
				change_sign(3);
		
		if (!(PIND & (1 << PD6)))
				change_sign(4);
		
		if (!(PINA & (1 << PA7)))
				change_sign(5);
		
		if (!(PINC & (1 << PC6)))
				change_sign(6);
		
		if (!(PINC & (1 << PC5)))
				change_sign(7);
		
		if (!(PINC & (1 << PC4)))
				change_sign(8);

		if (!(PINC & (1 << PC3)))
				change_sign(9);
		
		if (!(PINC & (1 << PC2)))
				change_sign(10);
		
		if (!(PINC & (1 << PC1)))
				change_sign(11);	
	}
}

int main(void)
{
	lcd_init();
	lcd_clear();
	
	// Butoane setate ca intrare + rezistenta de pull-up
	PORTD = 0x00;
	DDRD &= ~((1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3)
		| (1 << PD4) | (1 << PD5) | (1 << PD6) | (1 << PD7));
    PORTD |= ((1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3)
		| (1 << PD4) | (1 << PD5) | (1 << PD6) | (1 << PD7));
		
	PORTC = 0x00;
	DDRC &= ~((1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3)
		| (1 << PC4) | (1 << PC5));
    PORTC |= ((1 << PC6) | (1 << PC1) | (1 << PC2) | (1 << PC3)
		| (1 << PC4) | (1 << PC5));
		
	PORTA = 0x00;
	DDRA &= ~((1 << PA5) |(1 << PA6) | (1 << PA7));
    PORTA |= ((1 << PA5) |(1 << PA6) | (1 << PA7));
	
	title();	
	play();
}
